#include "ritual_ui.hpp"
#include "components.hpp"
#include "guecs.hpp"
#include "rand.hpp"

namespace gui {
  using namespace guecs;
  using std::any, std::any_cast, std::string, std::make_any;

  RitualUI::RitualUI(GameLevel level) :
    $level(level)
  {
    $gui.position(STATUS_UI_X, STATUS_UI_Y, STATUS_UI_WIDTH, STATUS_UI_HEIGHT);
    $gui.layout(
        "[_]"
        "[inv_slot5 | inv_slot6 | inv_slot7| inv_slot8]"
        "[inv_slot9 | inv_slot10 | inv_slot11| inv_slot12]"
        "[inv_slot13 | inv_slot14 | inv_slot15| inv_slot16]"
        "[inv_slot17 | inv_slot18 | inv_slot19| inv_slot20]"
        "[inv_slot21 | inv_slot22 | inv_slot23| inv_slot24]"
        "[*%(100,500)circle_area]"
        "[_]"
        "[_]"
        "[_]"
        "[_]"
        "[ ritual_ui ]");
  }

  void RitualUI::init() {
    // $gui.world().set_the<Background>({$gui.$parser});

    for(auto& [name, cell] : $gui.cells()) {
      auto button = $gui.entity(name);

      if(name == "circle_area") {
        // $gui.set<Rectangle>(button, {});
        $gui.set<Clickable>(button, {
            [this](auto, auto){ dbc::log("circle clicked"); }
        });
      } else if(name.starts_with("inv_slot")) {
        // $gui.set<Rectangle>(button, {});
        $gui.set<Clickable>(button, {
            [this, name](auto, auto){ dbc::log(fmt::format("inv_slot {}", name)); }
        });
      } else if(name == "ritual_ui") {
        $gui.set<Clickable>(button, {
            [this](auto, auto){ toggle(); }
        });
      }
    }

    $ritual_ui = textures::get("ritual_crafting_area");
    $ritual_ui.sprite->setPosition({0,0});
    $ritual_ui.sprite->setTextureRect($ritual_closed_rect);
    $ritual_state = RitualUIState::CLOSED;
    $ritual_anim.simple = false;
    $ritual_anim.looped = false;
    $ritual_anim.easing = ease::NONE;
    $ritual_anim.stationary = true;
    $ritual_anim.texture_width = 380;
    $ritual_anim.frames = 3;
    $ritual_anim.speed = 0.2f;

    $gui.init();
  }

  bool RitualUI::is_open() {
    return $ritual_state != RitualUIState::CLOSED;
  }

  bool RitualUI::mouse(float x, float y) {
    return $gui.mouse(x, y);
  }

  void RitualUI::toggle() {
    using enum RitualUIState;

    switch($ritual_state) {
      case OPEN:
        $ritual_state = CLOSING;
        break;
      case CLOSED:
        $ritual_state = OPENING;
        $ritual_anim.play();
        break;
      case OPENING: // ignored
        break;
      case CLOSING: // ignored
        break;
      default:
        dbc::sentinel("INVALID RitualUIState");
    }
  }

  /* WARNING: This is really not the greatest way to do this. */
  void RitualUI::update() {
    dbc::log("RITUAL UPDATE NOT IMPLEMENTED");
  }

  void RitualUI::render(sf::RenderWindow &window) {
    sf::IntRect rect;
    sf::Vector2f scale{1.0, 1.0};
    sf::Vector2f pos{0, 0};

    using enum RitualUIState;
    switch($ritual_state) {
      case OPEN: {
          rect = $ritual_open_rect;
        } break;
      case CLOSED: {
          rect = $ritual_closed_rect;
        }
        break;
      case OPENING: {
          if($ritual_anim.playing) {
            rect = $ritual_closed_rect;
            $ritual_anim.step(scale, pos, rect);
          } else {
            $ritual_state = OPEN;
            rect = $ritual_open_rect;
          }
        }
        break;
      case CLOSING: {
          rect = $ritual_closed_rect;
          $ritual_state = CLOSED;
        } break;
      default:
        dbc::sentinel("INVALID RitualUIState");
    }

    $ritual_ui.sprite->setTextureRect(rect);
    $ritual_ui.sprite->setPosition(pos);
    $ritual_ui.sprite->setScale(scale);

    window.draw(*$ritual_ui.sprite);
    if($ritual_state == OPEN) $gui.render(window);
  }
}
